use super::number::Number;


/// A trait that defines what is required to be considered
/// a Whole number. [List of types of numbers][1]
///
/// This is meant to have all the similarities between
/// [u8][2], [u16][3], [u32][4], [u64][5], and [usize][6].
/// Most of the comments and documentation were taken
/// from the [rust-lang std docs][7].
///
/// [1]: https://en.wikipedia.org/wiki/List_of_types_of_numbers
/// [2]: https://doc.rust-lang.org/std/primitive.u8.html
/// [3]: https://doc.rust-lang.org/std/primitive.u16.html
/// [4]: https://doc.rust-lang.org/std/primitive.u32.html
/// [5]: https://doc.rust-lang.org/std/primitive.u64.html
/// [6]: https://doc.rust-lang.org/std/primitive.usize.html
/// [7]: https://doc.rust-lang.org/std/index.html
pub trait Whole : Number
{
}


// Create a macro to ease typing and reading.
/// A macro to make implementing the trait easier for all the
/// base integer types in rust.
macro_rules! whole_trait_impl
{
   ($traitName: ident for $($varType: ty)*) =>
   ($(
      impl $traitName for $varType
      {
      }
   )*)
}


// Implement the trait for the types that are Whole numbers.
whole_trait_impl!(Whole for u8 u16 u32 u64 usize);
